package com.mobai.kafka.listener;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.mobai.domain.MqttServerModel;
import com.mobai.domain.Vehicle;
import com.mobai.forest.ForestGet;
import com.mobai.iotDB.service.IotDbServer;
import com.mobai.utils.RedisService;
import lombok.extern.slf4j.Slf4j;
import org.apache.iotdb.rpc.IoTDBConnectionException;
import org.apache.iotdb.rpc.StatementExecutionException;
import org.apache.kafka.clients.consumer.ConsumerRebalanceListener;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.OffsetAndMetadata;
import org.apache.kafka.clients.consumer.OffsetCommitCallback;
import org.apache.kafka.common.TopicPartition;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.kafka.annotation.KafkaListener;

import org.springframework.kafka.support.Acknowledgment;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.rmi.ServerException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * @description: kafka 消费者
 * @copyright: @Copyright (c) 2022
 * @company: hmblogs
 * @author: heming
 * @version: 1.0.0
 * @createTime: 2024-01-18 8:31
 */
@Component
@Slf4j
public class KafkaConsumerListenerExample {

    @Autowired
    private IotDbServer iotDbServer;

    @Autowired
    private ForestGet forestGet;

    /**
     * 监听消费
     *
     * @param record
     */
    @KafkaListener(topics = {"topic0", "topic1"}, groupId = "topics")
    public void consume(ConsumerRecord<String, String> record, Acknowledgment acknowledgment) {
        log.info("消费信息为：{}",record);
        // 无数据接口，存map集合
        Map<String,String> map = JSON.parseObject(record.value(),Map.class);
        // 存入iotDB
        try {
            iotDbServer.insertData(map);
            acknowledgment.acknowledge();
        } catch (StatementExecutionException e) {
            throw new RuntimeException(e);
        } catch (ServerException e) {
            throw new RuntimeException(e);
        } catch (IoTDBConnectionException e) {
            throw new RuntimeException(e);
        }
    }

    @Bean
    public void partitionConsumer() {
        ArrayList<TopicPartition> topicPartitions = new ArrayList<>();
        List<String> topics = forestGet.getIps().getData().stream().map(MqttServerModel::getTopic).toList();
        for (String topic : topics) {
            for (int i = 0; i < 8; i++) {
                TopicPartition topicPartition = new TopicPartition(topic, i);
                topicPartitions.add(topicPartition);
            }
        }
    }

    // 自定义分区分配监听器
    public static class MyPartitionAssignmentListener implements ConsumerRebalanceListener {
        @Override
        public void onPartitionsRevoked(Collection<TopicPartition> partitions) {
            // 可以在这里处理分区被撤销前的逻辑
            System.out.println("Partitions revoked: " + partitions);
        }

        @Override
        public void onPartitionsAssigned(Collection<TopicPartition> partitions) {
            // 可以在这里处理分区被分配后的逻辑
            System.out.println("Partitions assigned: " + partitions);
        }
    }

    /**
     * 解析kafka拉取数据
     *
     * @param jsonString
     * @return 车辆信息
     */
    public Vehicle getVehicle(String jsonString) {
        JSONObject jsonObject = JSONObject.parseObject(jsonString);
        // 解析车辆报文对象
        Vehicle vehicle = new Vehicle() {{
            setStartTime(Long.valueOf(jsonObject.get("startTime").toString()));
            setSpeed(new BigDecimal(jsonObject.get("speed").toString()));
            setLatitude(new BigDecimal(jsonObject.get("latitude").toString()));
            setLongitude(new BigDecimal(jsonObject.get("longitude").toString()));
            setMileage(new BigDecimal(jsonObject.get("mileage").toString()));
            setGear(jsonObject.get("gear").toString());
            setVehicleStatus(new BigDecimal(jsonObject.get("vehicleStatus").toString()).intValue());
            setChargingStatus(new BigDecimal(jsonObject.get("chargingStatus").toString()).intValue());
            setChargingEnergyStorageStatus(new BigDecimal(jsonObject.get("chargingEnergyStorageStatus").toString()).intValue());
            setEasStatus(new BigDecimal(jsonObject.get("easStatus").toString()).intValue());
            setMotorControllerTemperature(new BigDecimal(jsonObject.get("motorControllerTemperature").toString()));
            setTotalBatteryCurrent(new BigDecimal(jsonObject.get("totalBatteryCurrent").toString()));
            setSingleBatteryMaxVoltage(new BigDecimal(jsonObject.get("singleBatteryMaxVoltage").toString()));
            setOperatingStatus(new BigDecimal(jsonObject.get("operatingStatus").toString()).intValue());
            setHeatingStatus(new BigDecimal(jsonObject.get("heatingStatus").toString()).intValue());
            setDcdcStatus(new BigDecimal(jsonObject.get("dcdcStatus").toString()).intValue());
            setDriveMotorStatus(new BigDecimal(jsonObject.get("driveMotorStatus").toString()).intValue());
            setPositionStatus(new BigDecimal(jsonObject.get("positionStatus").toString()).intValue());
            setSocStatus(new BigDecimal(jsonObject.get("socStatus").toString()).intValue());
            setPtcStatus(new BigDecimal(jsonObject.get("ptcStatus").toString()).intValue());
            setEpsStatus(new BigDecimal(jsonObject.get("epsStatus").toString()).intValue());
            setAbsStatus(new BigDecimal(jsonObject.get("absStatus").toString()).intValue());
            setMcuStatus(new BigDecimal(jsonObject.get("mcuStatus").toString()).intValue());
            setBatteryInsulationStatus(new BigDecimal(jsonObject.get("batteryInsulationStatus").toString()).intValue());
            setBatteryStatus(new BigDecimal(jsonObject.get("batteryStatus").toString()).intValue());
            setChgStatus(new BigDecimal(jsonObject.get("chgStatus").toString()).intValue());
            setTotalBatteryVoltage(new BigDecimal(jsonObject.get("totalBatteryVoltage").toString()));
            setMotorSpeed(new BigDecimal(jsonObject.get("motorSpeed").toString()));
            setMotorCurrent(new BigDecimal(jsonObject.get("motorCurrent").toString()));
            setMotorVoltage(new BigDecimal(jsonObject.get("motorVoltage").toString()));
            setAccelerationPedal(new BigDecimal(jsonObject.get("accelerationPedal").toString()));
            setBrakePedal(new BigDecimal(jsonObject.get("brakePedal").toString()));
            setSelfCheckCounter(new BigDecimal(jsonObject.get("selfCheckCounter").toString()));
            setMotorTemperature(new BigDecimal(jsonObject.get("motorTemperature").toString()));
            setMaximumDischargePower(new BigDecimal(jsonObject.get("maximumDischargePower").toString()));
            setMaximumFeedbackPower(new BigDecimal(jsonObject.get("maximumFeedbackPower").toString()));
            setFuelConsumptionRate(new BigDecimal(jsonObject.get("fuelConsumptionRate").toString()));
            setVin(jsonObject.get("vin").toString());
            setVoltage(new BigDecimal(jsonObject.get("voltage").toString()));
            setCurrent(new BigDecimal(jsonObject.get("current").toString()));
            setResistance(new BigDecimal(jsonObject.get("resistance").toString()));
            setMotoTorque(new BigDecimal(jsonObject.get("motoTorque").toString()));
            setRemainingBattery(new BigDecimal(jsonObject.get("remainingBattery").toString()));
            setSingleBatteryMinVoltage(new BigDecimal(jsonObject.get("singleBatteryMinVoltage").toString()));
            setSingleBatteryMaxTemperature(new BigDecimal(jsonObject.get("singleBatteryMaxTemperature").toString()));
            setSingleBatteryMinTemperature(new BigDecimal(jsonObject.get("singleBatteryMinTemperature").toString()));
            setAvailableBatteryCapacity(new BigDecimal(jsonObject.get("availableBatteryCapacity").toString()));

        }};
        log.info("车辆报文信息 ： " + vehicle);
        return vehicle;
    }
}

